// experiments.h
// Implements systematic experiments to evaluate the schedulability ratio obtained with different analyses on randomly generated task sets

#pragma once

struct ExperimentParameters {
	// processing parameters
	bool parallel_mode = true; // false: run experiments sequentially; true: run experiments using multiple threads

	// experiment parameters
	double min_utilization = 0.0; // minimum value for the system utilization
	double max_utilization = 1.0; // maximum value for the system utilization
	double utilization_step = 0.05; // incremental step between system utilization points
	int num_iterations = 100; // number of task sets generated for each utilization point

	// generation parameters
	int num_tasks = 5; // number of tasks generated for each evaluated task set
	int min_period = 10; // minimum period generated for each task (T_min, with T_min<=T_max)
	int max_period = 100; // maximum period generated for each task (T_max, with T_min<=T_max)
	double min_susp_factor = 0.1; // value of beta_min parameter (0<=beta_min<=1)
	double max_susp_factor = 0.6; // value of beta_max parameter (0<=beta_max<=1)
	double deadline_factor = 1.0; // value of alpha parameter (0<=alpha<=1)
};

namespace experiments {
	// Performs the experiment based on the given parameters
	void sweep_utilization(ExperimentParameters exp_parameters);

	// Runs unit tests for this module
	bool test();
}
