// main.cpp
// Runs the experimental evaluation based on the arguments provided as input to the program

#include "test.h"
#include "experiments.h"
#include <iostream>
#include <sstream>

int main(int argc, char* argv[])
{
	if (test::run_tests()) {
		std::cout << "All tests succeeded" << std::endl;
	}

	ExperimentParameters exp_parameters;
	
	if (argc != 12) {
		std::cerr << "usage: dss-edf pmode umin umax ustep niter ntasks tmin tmax smin smax dlfac" << std::endl;
		std::cerr << "\tpmode:\tprocessing mode for the experiments (0: sequential, 1: multithreaded)" << std::endl;
		std::cerr << "\tumin, umax:\tminimum and maximum system utilization values (0<=umin<=umax<=1)" << std::endl;
		std::cerr << "\tustep:\tstep between the evaluated system utilization values" << std::endl;
		std::cerr << "\tniter:\tnumber of evaluated task sets for each utilization value" << std::endl;
		std::cerr << "\tntasks:\tnumber of tasks generated for each task set" << std::endl;
		std::cerr << "\ttmin, tmax:\tminimum and maximum period generated for each task (tmin<=tmax)" << std::endl;
		std::cerr << "\tsmin, smax:\tvalues of the beta_min and beta_max parameters (0<=smin<=smax<=1)" << std::endl;
		std::cerr << "\tdlfac:\tvalue of the alpha parameter (0<=dlfac<=1)" << std::endl;
		return -1;
	}

	std::istringstream input;
	input.str(argv[1]);
	if (input >> exp_parameters.parallel_mode) {
	}
	else {
		return -1;
	}
		
	input.clear();
	input.str(argv[2]);
	if (input >> exp_parameters.min_utilization) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[3]);
	if (input >> exp_parameters.max_utilization) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[4]);
	if (input >> exp_parameters.utilization_step) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[5]);
	if (input >> exp_parameters.num_iterations) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[6]);
	if (input >> exp_parameters.num_tasks) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[7]);
	if (input >> exp_parameters.min_period) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[8]);
	if (input >> exp_parameters.max_period) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[9]);
	if (input >> exp_parameters.min_susp_factor) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[10]);
	if (input >> exp_parameters.max_susp_factor) {
	}
	else {
		return -1;
	}

	input.clear();
	input.str(argv[11]);
	if (input >> exp_parameters.deadline_factor) {
	}
	else {
		return -1;
	}

	std::cout << "Running experiments..." << std::endl;
	experiments::sweep_utilization(exp_parameters);

	return 0;
}
